
/* --------------------------------------------------------------------------
   A. Veneziani - esempio basato sull'uso di oggetti dinamici "Persona"

                    1 - ogni oggetto  istanziato e contiene i dati sottoposti
                        a statistica (altezza e eta')
                    2 - La classe contiene anche un oggetto clona che effettua
                        copie di oggetti con le stesse caratteristiche
                    3 - Si  fatto inoltre ricorso ad un vettore di puntatori
                        ad oggetti del tipo considerato.

  --------------------------------------------------------------------------  */

#include "conio.h"
#include <iostream>

using namespace std;

#pragma hdrstop

//---------------------------------------------------------------------------

#pragma argsused

// dichiara una classe Persona, atta a contenere dei dati
// relativi ad una persona (altezza, eta', altro...)
class Persona {

    int altezza;     // dati intrinseci all'oggetto (- attributi -)
    int eta;

    public:

    // - metodo per clonare l' oggetto persona -
    // il nuovo oggetto  del tutto simile al precedente,
    // ha solo l'area di memoria allocata ed il puntatore ad essa differenti
    // (in quanto oggetto distinto)
    Persona* clona()
    {
        Persona *pc;

        pc = new Persona();
        pc->altezza = this->altezza;
        pc->eta = this->eta;

        return(pc);
    }

    // metodi per leggere altezza ed eta
    void fissa_altezza(int h)
    {
       altezza = h;
    }

    void fissa_eta(int age)
    {
        eta = age;
    }

    // metodi che rendono, quando richiesto, altezza ed eta' 
    int leggi_altezza()
    {
       return(altezza);
    }

    int leggi_eta()
    {
       return(eta);
    }
};

int main(int argc, char* argv[])
{
    int cc, n_tot_pers, npers, h, eta, nrif;
    int hmax, hmin, etamin, etamax, sh, seta;
    float hmed, etamed;
    Persona *pers[100];

    // "azzeramento" di tutti i puntatori alle varie
    // istanze di persona
    for (cc = 0; cc < 100; cc++)  pers[cc] = NULL;

    // ciclo di lettura dei dati delle varie persone
    n_tot_pers = 0;
    do {
       // se vi sono dati coincidenti...
       // a questa richiesta viene inserito npers <> 1    
       cout << "Numero di persone:  ";
       cin >> npers;

       // l'indicazione 0 indica la fine degli inserimenti...
       if (npers > 0)
       {
           // se non e'  numero persone non e' 0 si
           // inseriscono gli altri dati
           cout << "Altezza:  ";
           cin >> h;
           cout << "Eta':  ";
           cin >> eta;

           // si alloca un primo oggetto e si inseriscono
           // in essi i dati letti
           pers[n_tot_pers] = new Persona();
           pers[n_tot_pers]->fissa_eta(eta);
           pers[n_tot_pers]->fissa_altezza(h);

           // prende nota del puntatore che punta al primo
           // oggetto della possibile serie
           nrif = n_tot_pers;

           n_tot_pers++;

           // clonazione di altri possibili oggetti
           // analoghi al primo
           for (cc = 1; cc < npers; cc++)
           {
              pers[n_tot_pers] = pers[nrif]->clona();
              n_tot_pers++;              
           }
       }   

    }  while(npers > 0);
    // esce quando l'indicazione del numero di persone e' 0

    // calcolo dei massimi dei minimi e delle medie dei valori
    // presenti nei vari oggetti allocati, a cui si accede
    // tramite i puntatori contenuti nel vettore pers[].
    sh = 0;
    seta = 0;
    for (cc = 0; cc < n_tot_pers; cc++)
    {
        // lettura dell'altezza dall'oggetto
        h = pers[cc]->leggi_altezza();
        // somma delle altezze per calcolare la media
        sh = sh + h;

        // ricerca della minima e massima altezza 
        if (cc == 0)
        {
           hmax = h;  hmin = h;
        }
        else
        {
           if (h > hmax)  hmax = h;
           if (h < hmin)  hmin = h;
        }

        // operazioni analoghe a sopra che coinvolgono le eta'
        eta = pers[cc]->leggi_eta();
        seta = seta + eta;

        if (cc == 0)
        {
           etamax = eta;   etamin = eta;
        }
        else
        {
            if (eta > etamax)  etamax = eta;
            if (eta < etamin)  etamin = eta;
        }
    }

    // calcolo delle medie dei valori di eta e altezza
    etamed = seta / ((float) n_tot_pers);
    hmed = sh / ((float) n_tot_pers);

    // output dei risultati
    cout << endl << endl;
    cout << "Statistiche sulla popolazione ----------------------------" << endl;
    cout << "Eta' massima della popolazione:   " << etamax <<endl;
    cout << "Eta' minima della popolazione:   " << etamin <<endl;
    cout << "Eta' media della popolazione:   " << etamed <<endl;
    cout << "Altezza massima della popolazione:   " << hmax <<endl;
    cout << "Altezza minima della popolazione:   " << hmin <<endl;
    cout << "Altezza media della popolazione:   " << hmed <<endl;
    cout << "---------------------------------------------------------" << endl;

    getch();

    return 0;
}
//---------------------------------------------------------------------------

